import requests

import re, json
from datetime import datetime, timezone, timedelta

import functions

JST = timezone(timedelta(hours = 9))

class YahooAuctionsItem:
	"""
	A class for handling items on Yahoo! Auctions Japan


	Attributes
	----------
	name : str
		The name given to the auction item by the user.

	original_name : str
		The name as it appears on Y!A,

	favourite : bool
		Whether or not the user has "favourited" this item.

	url : bool
		The URL this item was added from. If not provided, defaults to https://buyee.jp/item/yahoo/auction/{id}.

	id : str
		The ID of the item, found at the end of the URL.

	last_checked : datetime
		The time that the item's status was last checked by buypeeb.

	available : bool
		Whether or not the item is still available (if the auction is still available)

	ready : bool
		Whether or not the object is ready. If not, some of the data (price, available, etc.) may be out of date or unset.

	updating : bool
		If true, the object is currently being updated. Should never be true if ready is true.

	price : float
		The price of the item in yen.

	bids : int
		The number of bids that have been placed on the item.

	start_date : datetime
		The start date and time of the item in JST.

	end_date : datetime
		The end date and time of the item in JST.
	"""
	def __init__(self, url: str, id: str, name: str = None, from_json: dict = None):
		# note - incoming url is not validated in any way!
		self.name = name
		self.price = 0
		self.original_name = None
		self.favourite = None
		self.end_date = None

		if url == None and from_json != None and id != None:
			self.id = id
			self.name = from_json['name']
			self.original_name = from_json['original_name']
			self.favourite = from_json['favourite']
			self.url = f"https://buyee.jp/item/yahoo/auction/{self.id}"

		elif url != None and id != None:
			self.url = url
			self.id = id

		else:
			raise RuntimeError

		self.last_checked = datetime.fromisoformat('1970-01-01')
		self.available = True
		self.ready = False
		self.updating = False

	def update(self):
		self.updating = True
		try:
			# the good news is, yahoo japan returns all the data we need in handy json format
			# the bad news is that the only way to get that json format is to download the whole auction page and grep it

			# r = requests.get(f"https://page.auctions.yahoo.co.jp/jp/auction/{self.id}").text
			r = open("yahoo.html").read()
			j = json.loads(re.match(r'.*var pageData ?= ?(\{.*?\});', r, re.DOTALL).group(1))
		except:
			raise

		j = j['items']
		self.price = float(j['price'])
		self.bids = j['bids']
		self.start_date = datetime.fromisoformat(j['starttime']).replace(tzinfo = JST)
		self.end_date = datetime.fromisoformat(j['endtime']).replace(tzinfo = JST)
		self.last_checked = datetime.now(JST)
		self.original_name = j['productName']
		if self.name == None:
			self.name = j['productName']

		self.ready = True
		self.updating = False

	def price_jpy(self):
		return f"¥{self.price:.2f}"

	def price_aud(self, rate = 75.0):
		return f"${self.price / rate:.2f}"

	def ending_in(self):
		return "heenlo"
